import datetime
import sys

from django.conf import settings
from django.http import HttpResponseRedirect, HttpResponse, HttpResponseForbidden
from django.core.urlresolvers import reverse, resolve
from django.contrib.auth.models import AnonymousUser, User
from django.contrib.auth import login, get_backends

from views import get_tokens_from_cookie
from signals import signal_token_visited
from models import ProtectedURL, Token


class ProtectedURLMiddleware(object):
    """
    This middleware requires a login or associated token for protected urls.
    """

    def check_for_user_or_token(self, request):
        # check user is not anonymous and is logged in
        if not isinstance(request.user, AnonymousUser) and request.user.is_authenticated():
            return True
        # see if a protected url token exists
        if getattr(request, 'valid_token', None):
            return True
        return False
    
    def process_request(self, request):
        CHECK_TOKEN = False
        try:
            view, args, kwargs = resolve(request.path)
            if 'use_token_url' is view.__name__: CHECK_TOKEN = True
        except:
            pass
        if CHECK_TOKEN:
            pass
        elif request.path == reverse('login_form'):
            pass
        elif request.path is '/':
            where_sql = 'SUBSTR(%s, 1, LENGTH(url)) = url'
            if ProtectedURL.objects.extra(where=[where_sql], params=[request.path]):
                user_tokens = get_tokens_from_cookie(request) # get the user's tokens
                tokens = Token.active_objects.filter(token__in=user_tokens).order_by('url')
                if tokens:
                    for token in tokens:
                        if request.path.startswith(token.url):
                            signal_token_visited.send(sender=self.__class__, request=request, token=token)
                            request.valid_token = token
                            break
                allowed = self.check_for_user_or_token(request)
                if not allowed:
                    return HttpResponseRedirect(reverse('login_form'))
        else:
            if ProtectedURL.objects.get(url='/'):
                user_tokens = get_tokens_from_cookie(request) # get the user's tokens
                tokens = Token.active_objects.filter(token__in=user_tokens, url__exact='/')
                if tokens:
                    request.valid_token = tokens[0]
                allowed = self.check_for_user_or_token(request)
                if not allowed:
                    return HttpResponseRedirect(reverse('login_form'))

class TokenAuthLoginMiddleware(object):
    def process_request(self, request):
        if isinstance(request.user, AnonymousUser):
            token = getattr(request, 'valid_token', None) # found from cookies (valid)
            if token:
                try:
                    user = User.objects.get(email=token.email)
                    backend = get_backends()[0]
                    user.backend = "%s.%s" % (backend.__module__, backend.__class__.__name__)
                    login(request, user)
                except User.DoesNotExist:
                    pass
                    
            


from django.conf.urls.defaults import *

urlpatterns = patterns('',

    url('^token_auth/',
        include('token_auth.urls')
    ),
    
    url('^protected/$',
        'token_auth.tests.views.example'
    ),
    url('^protected/sub1/$',
        'token_auth.tests.views.example'
    ),
    url('^protected/sub1/sub2/$',
        'token_auth.tests.views.example'
    ),
    url('^login/$',
        'token_auth.tests.views.example',
        name='login_form'
    )
    
)



from django.conf.urls.defaults import *
from django.contrib.auth.views import login, logout, password_change, password_change_done
from django.contrib import admin


urlpatterns = patterns('',
    url('^create/$',
        'token_auth.views.create_protected_url', name='create_protected_url' ),
    url('^created/$',
        'token_auth.views.protected_url_created', name='protected_url_created'),
    url('^(?P<token_str>\w+)/$',
        'token_auth.views.use_token_url', name='use_token_url' ),
    url('^(?P<token_str>\w+)/forward/$',
        'token_auth.views.forward_token', name='forward_token_url' ),
    url('^(?P<token_str>\w+)/expire/$',
        'token_auth.views.expire_token_url', name='expire_token_url' ),
    url('^(?P<token_str>\w+)/used/$',
        'token_auth.views.token_url_used', name='token_url_used' ),
    url('^(?P<token_str>\w+)/expired/$',
        'token_auth.views.token_url_expired', name='token_url_expired' ),
    url('^(?P<token_str>\w+)/invalid/$',
        'token_auth.views.token_url_invalid', name='token_url_invalid' ),
)



import django.dispatch


""" Signal when a token is used (link in email is visited)"""
signal_token_used = django.dispatch.Signal(providing_args=["request", "token"])


""" Signal when a url with token a is visited (the url the token is connected to)"""
signal_token_visited = django.dispatch.Signal(providing_args=["request", "token"])




from django import forms
from models import Token


class TokenForm(forms.Form):
    url = forms.CharField(max_length=255)
    valid_until = forms.DateField(required=False)
    emails = forms.CharField(max_length=255)
    forward_count = forms.IntegerField(required=False)
    
    def clean_emails(self):
        emails = self.cleaned_data['emails'].split(';')
        return emails

        
class TokenAddForm(forms.ModelForm):
    class Meta:
        model = Token
        fields = ('name', 'email', 'forward_count')


class ForwardProtectedURLForm(forms.Form):
    def __init__(self, token, *args, **kwargs):
        super(ForwardProtectedURLForm, self).__init__(*args, **kwargs)
        self.token = token
    
    emails = forms.CharField(max_length=255)
    
    def clean_emails(self):
        emails = self.cleaned_data['emails'].split(';')
        if self.token.forward_count and len(emails) > self.token.forward_count:
                raise forms.ValidationError("You can only forward to %i emails." % (self.token.forward_count or 0))
        return emails



from django.test import TestCase
from django.test.client import Client
from django.conf import settings
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.utils.encoding import force_unicode
import datetime

from token_auth.models import ProtectedURL, Token
from token_auth.views import TOKEN_COOKIE

EMAILS = 'test@test.com;test2@test.com'

FORM_DATA_URL_1 = { 'url': '/protected/', 'valid_until': '', 'emails': EMAILS }
FORM_DATA_URL_2 = { 'url': '/protected/sub1/', 'valid_until': '', 'emails': EMAILS }
FORM_DATA_URL_3 = { 'url': '/protected/sub1/sub2/', 'valid_until': '', 'emails': EMAILS }

FORM_DATA_FORWARD_1 = { 'emails': EMAILS }
FORM_DATA_FORWARD_2 = { 'emails': 'test@test.com' }


class TestURLs(TestCase):

    urls = 'token_auth.tests.urls'

    fixtures = ['test_fixtures.json']


    def setUp(self):
        url = ProtectedURL.objects.create(url='/protected/')


    def login(self, client, password='password'):
        login = client.login(username='token_auth', password=password)
        self.failUnless(login, 'Could not log in')


    def testAddURL(self):

        client = Client()

        response = client.get(reverse('create_protected_url'))
        self.failUnlessEqual(response.status_code, 302)

        self.login(client)

        response = client.get(reverse('create_protected_url'))

        self.failUnlessEqual(response.status_code, 200)
        self.failUnless(response, "no form in context")
        self.failUnlessEqual(response.context['form'].errors, {})

        response = client.post(reverse('create_protected_url'), FORM_DATA_URL_1)
        self.failUnlessEqual(response.status_code, 302)

        response = client.post(reverse('create_protected_url'), FORM_DATA_URL_2)
        self.failUnlessEqual(response.status_code, 302)

        response = client.post(reverse('create_protected_url'), FORM_DATA_URL_3)
        self.failUnlessEqual(response.status_code, 302)


    def testVisitURL302(self):

        client = Client()
        
        # test that protection works
        
        response = client.get('/protected/')
        self.failUnlessEqual(response.status_code, 302)

        response = client.get('/protected/sub1/')
        self.failUnlessEqual(response.status_code, 302)

        response = client.get('/protected/sub1/sub2/')
        self.failUnlessEqual(response.status_code, 302)


    def testVisitURL200Cookie(self):

        url = '/protected/'

        token = Token(url=url)
        token.save()

        client = Client()

        # test that tokens work
        response = client.get(token.use_token_url())
        self.failUnlessEqual(response.status_code, 302)
        self.failUnlessEqual(client.cookies[TOKEN_COOKIE].value, token.token)

        response = client.get("/protected/")
        self.failUnlessEqual(response.status_code, 200)

        response = client.get("/protected/sub1/")
        self.failUnlessEqual(response.status_code, 200)

        response = client.get("/protected/sub1/sub2/")
        self.failUnlessEqual(response.status_code, 200)

        response = client.get(token.use_token_url())
        self.failUnlessEqual(response.status_code, 302)

        # test for two tokens
        token2 = Token(url=url)
        token2.save()

        response = client.get(token2.use_token_url())
        self.failUnlessEqual(response.status_code, 302)
        self.failUnless(client.cookies[TOKEN_COOKIE].value, token.token + '|' + token2.token)

        token.delete()
        token2.delete()

        # test for expired tokens
        token3 = Token(url=url)
        token3.save()

        response = client.get(token3.use_token_url())
        self.failUnlessEqual(response.status_code, 302)

        response = client.get("/protected/")
        self.failUnlessEqual(response.status_code, 200)

        token3.valid_until = datetime.datetime.today() - datetime.timedelta(days=2)
        token3.save()

        response = client.get("/protected/")
        self.failUnlessEqual(response.status_code, 302)


    def testVisitURL200User(self):

        client = Client()

        self.login(client)

        response = client.get('/protected/')
        self.failUnlessEqual(response.status_code, 200)

        response = client.get('/protected/sub1/')
        self.failUnlessEqual(response.status_code, 200)

        response = client.get('/protected/sub1/sub2/')
        self.failUnlessEqual(response.status_code, 200)


    def testForwardToken(self):

        client = Client()

        # test forwarding of token
        url = '/protected/'

        token = Token(url=url)
        token.save()

        response = client.get(token.use_token_url())
        self.failUnlessEqual(response.status_code, 302)

        response = client.get(token.forward_token_url())
        self.failUnlessEqual(response.status_code, 200)
        self.failUnlessEqual(response.context['token'].can_forward, False)
        self.failUnlessEqual(force_unicode(response.context['error']), 'Apologies! This token can not be forwarded.')

        token.delete()

        token = Token(url=url, forward_count=None)
        token.save()

        response = client.get(token.use_token_url())
        self.failUnlessEqual(response.status_code, 302)

        response = client.get(token.forward_token_url())
        self.failUnlessEqual(response.context['token'].can_forward, True)
        self.failUnlessEqual(force_unicode(response.context['error'], strings_only=True), None)

        response = client.post(token.forward_token_url(), FORM_DATA_FORWARD_1)
        self.failUnlessEqual(response.status_code, 302)

        token.delete()

        # test max number of forwards
        url = '/protected/'
        token = Token(url=url, forward_count=3)
        token.save()
        
        response = client.get(token.use_token_url())
        response = client.get(token.forward_token_url())
        self.failUnlessEqual(force_unicode(response.context['error'], strings_only=True), None)

        response = client.post(token.forward_token_url(), FORM_DATA_FORWARD_1)
        self.failUnlessEqual(response.status_code, 302)

        # grab token from db
        token = Token.objects.get(pk=token.pk)

        self.failUnlessEqual(token.forward_count, 1)

        response = client.post(token.forward_token_url(), FORM_DATA_FORWARD_1)
        self.failUnlessEqual(response.status_code, 200)

        # grab token from db
        token = Token.objects.get(pk=token.pk)
        self.failUnlessEqual(token.forward_count, 1)

        response = client.post(token.forward_token_url(), FORM_DATA_FORWARD_2)
        self.failUnlessEqual(response.status_code, 302)

        # grab token from db
        token = Token.objects.get(pk=token.pk)
        self.failUnlessEqual(token.forward_count, 0)


    def testTokenAuthLogin(self):

        client = Client()

        settings.MIDDLEWARE_CLASSES = list(settings.MIDDLEWARE_CLASSES) + ['token_auth.middleware.TokenAuthLoginMiddleware']

        from django.contrib.auth.models import User
        user = User.objects.get(pk=1)

        url = '/protected/'
        token = Token(url=url, email=user.email)
        token.save()

        response = client.get(token.use_token_url())
        self.failUnlessEqual(response.status_code, 302)

        response = client.get('/protected/')
        self.failUnlessEqual(response.status_code, 200)

        token.delete()

        response = client.get('/protected/')
        self.failUnlessEqual(response.status_code, 200)



from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from models import ProtectedURL, Token
from views import forward_token
from forms import TokenAddForm


class TokenInline(admin.TabularInline):
    model = Token
    form = TokenAddForm
    extra = 1


class ProtectedURLAdmin(admin.ModelAdmin):
    #    inlines = [TokenInline, ]
    list_display = ('url', )
admin.site.register(ProtectedURL, ProtectedURLAdmin)


class TokenAdmin(admin.ModelAdmin):
    actions = ['send_token_email', ]
    list_display = ('email', 'url', 'token', 'valid_until', 'used')
    fieldsets = (
        (None, {
            'fields': (('url', 'valid_until')),
            'description': _('Create or select a url to protect and optionally set an expiration date.')
        }),
        (_('Visitor Information'), {
            'fields': (('name', 'email'), 'forward_count'),
            'description': _('Enter a name and email for the person you want to associate with this token and how many times they can forward it.')
        })
    )

    def send_token_email(self, request, queryset):
        """
        Sends token email(s) for the selected users.

        """
        for token in queryset:
            if not token.expired: forward_token(token)
    send_token_email.short_description = _("Send token email(s).")
admin.site.register(Token, TokenAdmin)



from datetime import datetime
from django.db.models import Manager, Q


class ActiveTokenManager(Manager):
    """
    Manager for active tokens (not expired).
    """
    def get_query_set(self):
        queryset = super(ActiveTokenManager, self).get_query_set()
        return queryset.filter(Q(valid_until__isnull=True) | Q(valid_until__gte=datetime.now))



from datetime import datetime

from django.db import models
from django.db.models import permalink
from django.conf import settings
from django.core.mail import EmailMessage
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _

from managers import ActiveTokenManager


class ProtectedURL(models.Model):
    """
    Model to identify protected URLs.
    """
    url = models.CharField(_('Protected URL'), max_length=255, unique=True)

    class Meta:
        verbose_name = _('Protected URL')
        verbose_name_plural = _('Protected URLs')

    def __unicode__(self):
        return u"%s" % self.url

    objects = models.Manager()


class Token(models.Model):
    """
    Model to store tokens related to individual urls and an email address.
    These tokens can expire and can optionally be forwarded by the user
    a definable number of times.
    """
    url = models.CharField(_('Allow URL'), max_length=255)
    name = models.CharField(_('Name'), max_length=64, blank=True, null=True)
    email = models.EmailField(_('Email Address'))
    token = models.CharField(_('URL Token'), max_length=20, editable=False)
    valid_until = models.DateTimeField(_('Valid Until'), null=True, blank=True)
    forward_count = models.PositiveIntegerField(_('Forward Count'), null=True, blank=True, default=0)
    used = models.BooleanField(_('Token Used?'), default=False)
    single_use = models.BooleanField(_('Single Use Token?'), default=False)


    objects = models.Manager()
    active_objects = ActiveTokenManager()


    class Meta:
        verbose_name = _('Protected URL Token')
        verbose_name_plural = _('Protected URL Tokens')
        ordering = ('email', '-used', '-valid_until', 'url')
    
    def __unicode__(self):
        return u"%s:%s - %s" % (self.email, self.url, self.token)
    
    def create_token(self):
        """
        Create a unique SHA token/hash using the project SECRET_KEY, URL,
        email address and current datetime.
        """
        from django.utils.hashcompat import sha_constructor
        hash = sha_constructor(settings.SECRET_KEY + self.url + self.email + unicode(datetime.now()) ).hexdigest()
        return hash[::2]

    """ if token doesn't exist create it on save """
    def save(self, **kwargs):
        if not self.pk:
            self.token = self.create_token()
        super(Token, self).save(**kwargs)
    
    def _forward_token_url(self):
        return ('forward_token_url', (), {'token_str': self.token})
    forward_token_url = permalink(_forward_token_url)
    
    def _use_token_url(self):
        return ('use_token_url', (), {'token_str': self.token})
    use_token_url = permalink(_use_token_url)

    """ make the url, token and associated email immutable """
    def __setattr__(self, name, value):
        if name in ['token', 'email', 'url']:
            if getattr(self, name, None): return
        super(Token, self).__setattr__(name, value)

    def _get_expired_boolean(self):
        """
        Returns ``True`` if the token has expired.
        """
        return self.valid_until is None or self.valid_until >= datetime.datetime.now()
    expired=property(_get_expired_boolean)

    def _get_forward_boolean(self):
        """
        Returns ``True`` if the token can be forwarded.
        """
        return self.forward_count is None or self.forward_count is not 0
    can_forward = property(_get_forward_boolean)

    def send_token_email(self, forwarded_by=None):
        subject = render_to_string('token_auth/token_email_subject.txt', { 'restricted_url': self.url } )
        subject = ''.join(subject.splitlines())
        message = render_to_string('token_auth/token_email_message.txt', { 'token': self, 'forwarded_by': forwarded_by } )
        if not settings.DEBUG:
            if not self.expired:
                EmailMessage(subject, message, [self.email] ).send()



import time, datetime

from django.views.generic.simple import direct_to_template
from django.contrib.auth.decorators import login_required
from django.template.loader import render_to_string
from django.conf import settings
from django.core.mail import EmailMessage
from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.utils.http import cookie_date
from django.conf import settings
from django.contrib.auth.models import AnonymousUser
from django.utils.translation import ugettext_lazy as _
from django.shortcuts import render_to_response, get_object_or_404

from forms import TokenForm, ForwardProtectedURLForm
from models import Token, ProtectedURL
from signals import signal_token_used


TOKEN_COOKIE = 'protectedurltokens'


def get_tokens_from_cookie(request):
    tokens = request.COOKIES.get(TOKEN_COOKIE, None)
    tokens_list = (tokens and tokens.split('|') or [])
    tokens_list = list(set(tokens_list))
    return tokens_list

def user_has_token_cookie(request, token=None):
    if not token is None:
        tokens = request.COOKIES.get(TOKEN_COOKIE, '')
        tokens_list = (tokens and tokens.split('|') or [])
        if token in tokens_list:
            return True
    return False


@login_required
def create_protected_url(request, **kwargs):
    kwargs['extra_context'] = {}
    if request.method == 'POST':
        form = TokenForm(request.POST)
        if form.is_valid():
            for email in form.cleaned_data['emails']:
                token = Token(
                    url=form.cleaned_data['url'],
                    valid_until=form.cleaned_data['valid_until'],
                    forward_count=form.cleaned_data['forward_count'],
                    email=email
                )
                token.save()
                subject = render_to_string('token_auth/token_email_subject.txt', { 'token': token } )
                subject = ''.join(subject.splitlines())
                message = render_to_string('token_auth/token_email_message.txt', { 'token': token } )
                if not settings.DEBUG:
                    EmailMessage(subject, message, [email] ).send()
                return HttpResponseRedirect(reverse('protected_url_created'))
    else:
        form = TokenForm(initial={'url': request.GET.get('url', '')})
    kwargs['extra_context']['form'] = form
    return direct_to_template(request, template='token_auth/create_protected_url.html', **kwargs)


def forward_token(request, token_str=None, **kwargs):
    kwargs['extra_context'] = {}
    error = None
    token = get_object_or_404(Token, token=token_str)
    user_tokens = get_tokens_from_cookie(request)
    if not token.can_forward:
        error = _("Apologies! This token can not be forwarded.")
    else:
        if request.user.is_staff:
            pass
        elif not token.token in user_tokens:
            error = _("Apologies! You are not allowed to forward this token.")
    kwargs['extra_context']['token'] = token
    kwargs['extra_context']['error'] = error
    if not error:
        if request.method == 'POST':
            form = ForwardProtectedURLForm(token, request.POST)
            if form.is_valid():
                if token.forward_count:
                    token.forward_count = token.forward_count - len(form.cleaned_data['emails'])
                    token.save()
                for email in form.cleaned_data['emails']:
                    forwarded_token = Token( url=token.url, valid_until=token.valid_until, forward_count=0, email=email )
                    forwarded_token.save()
                    forwarded_token.send_token_email()
                return HttpResponseRedirect(reverse('protected_url_created'))
        else:
            form = ForwardProtectedURLForm(token)
        kwargs['extra_context']['form'] = form
    return direct_to_template(request, template='token_auth/forward_token.html', **kwargs)



def use_token_url(request, token_str=None, **kwargs):
    if not token_str is None:
        token = get_object_or_404(Token, token=token_str)
        # ok we have a token so let's start setting up the response
        response = HttpResponseRedirect(token.url)
        # if it's not used yet, set it as used
        if not token.used:
            response = HttpResponseRedirect(token.url)
            token.used = True
            token.save()
            signal_token_used.send(sender=use_token_url, request=request, token=token)
            max_age = 2592000
            expires_time = time.time() + max_age
            expires = cookie_date(expires_time)
            tokens_list = list(set(get_tokens_from_cookie(request) + [token.token]))
            tokens = '|'.join(tokens_list)
            response.set_cookie(TOKEN_COOKIE, tokens, max_age=max_age, expires=expires)
        # if token is used but user doesn't have token cookie so tell them NO
        elif not user_has_token_cookie(request, token=token.token):
            response = HttpResponseRedirect(reverse('token_url_used'))
        # cookie's expired... answer is still no
        elif not token.valid_until is None and token.valid_until <= datetime.datetime.now():
            response = HttpResponseRedirect(reverse('token_url_expired'))
        # user has a cookie with that token and it's still valid
        elif token.single_use:
            token.delete()
        return response
    else:
        return direct_to_template(request, template='token_auth/token_url_invalid.html', **kwargs)


def expire_token_url(request, token=None, **kwargs):
    max_age = 2592000
    expires_time = time.time() - max_age
    expires = cookie_date(expires_time)
    response = HttpResponseRedirect(reverse('token_url_expired'))
    response.set_cookie(TOKEN_COOKIE, '', max_age=max_age, expires=expires)
    return response


def token_url_used(request, template='token_auth/token_url_used.html', **kwargs):
    return direct_to_template(request, template, **kwargs)


def token_url_expired(request, template='token_auth/token_url_expired.html', **kwargs):
    return direct_to_template(request, template, **kwargs)


def token_url_invalid(request, template='token_auth/token_url_invalid.html', **kwargs):
    return direct_to_template(request, template, **kwargs)


def protected_url_created(request, template='token_auth/protected_url_created.html', **kwargs):
    return direct_to_template(request, template, **kwargs)



# Django settings for cms project.

DEBUG = True
TEMPLATE_DEBUG = DEBUG

ADMINS = (
    # ('Your Name', 'your_email@domain.com'),
)

CACHE_BACKEND = 'locmem:///'

MANAGERS = ADMINS

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be avilable on all operating systems.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'America/Los_Angeles'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# Absolute path to the directory that holds media.
MEDIA_URL = '/media/uploaded/'
ADMIN_MEDIA_PREFIX = '/media/admin/'

# Make this unique, and don't share it with anybody.
SECRET_KEY = '*xq7m@)*f2awoj!spa0(jibsrz9%c0d=e(g)v*!17y(vx0ue_3'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.load_template_source',
    'django.template.loaders.app_directories.load_template_source',
#     'django.template.loaders.eggs.load_template_source',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    "django.core.context_processors.auth",
    "django.core.context_processors.i18n",
    "django.core.context_processors.debug",
    "django.core.context_processors.request",
    "django.core.context_processors.media",
)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.middleware.doc.XViewMiddleware',
    'django.contrib.csrf.middleware.CsrfMiddleware',
    'token_auth.middleware.ProtectedURLMiddleware',
)

ROOT_URLCONF = 'token_auth.urls'

INSTALLED_APPS = (
    'token_auth',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.admin',
    'django.contrib.sites',
)




import os
import sys
from os.path import abspath, dirname, join
from glob import glob
from distutils.command.install import INSTALL_SCHEMES
from setuptools import setup, find_packages


for scheme in INSTALL_SCHEMES.values():
    scheme["data"] = scheme["purelib"]


data_files = [
    ["src/token_auth/templates/base_templates", 
    glob("src/token_auth/templates/base_templates/*.html")]
]

root_dir = abspath(os.path.dirname(__file__))
sys.path.append(os.path.join(root_dir, "src"))

version = __import__('token_auth').get_version()

def read(fname):
    return open(os.path.join(abspath(os.path.dirname(__file__)), fname)).read()

setup(
    name='django-token-auth',
    version = version,
    url = 'http://github.com/mogga/django-token-auth/',
    license = 'BSD',
    description = "app that provides limited authentication via hash-type URL.",
    long_description = '',

    author = 'Oyvind Saltvik, Robert Moggach',
    author_email = 'oyvind.saltvik@gmail.com, rob@moggach.com',

    packages = find_packages('src'),
    package_dir = {'': 'src'},
    data_files = data_files,

    install_requires = ['setuptools'],

    classifiers = [
        'Development Status :: 4 - Beta',
        'Programming Language :: Python',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: BSD License',
        'Topic :: Internet :: WWW/HTTP',
    ],

    keywords = 'python django hash authentication'

)



#v0.3.1b1
VERSION = (0, 3, 1, 'beta', 1)

STATUSES = {'alpha': 'a', 'beta': 'b', 'releasecandidiate': 'rc' }

def get_version():
    version = '%s.%s' % (VERSION[0], VERSION[1])
    
    if VERSION[2]:
        version = '%s.%s' % (version, VERSION[2])
    
    if VERSION[3:] == ('alpha', 0):
        version = '%s pre-alpha' % version
    
    else:
        if VERSION[3] != 'final':
            version = '%s%s%s' % (version, STATUSES[VERSION[3]], VERSION[4])
    return version
